'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _clusterize = require('clusterize.js');

var _clusterize2 = _interopRequireDefault(_clusterize);

var _flattree = require('flattree');

var _renderer = require('./renderer');

var _polyfill = require('./polyfill');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var extend = function extend(target) {
    for (var _len = arguments.length, sources = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        sources[_key - 1] = arguments[_key];
    }

    sources.forEach(function (source) {
        for (var key in source) {
            if (source.hasOwnProperty(key)) {
                target[key] = source[key];
            }
        }
    });

    return target;
};

var InfiniteTree = function (_events$EventEmitter) {
    _inherits(InfiniteTree, _events$EventEmitter);

    function InfiniteTree() {
        var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

        _classCallCheck(this, InfiniteTree);

        // Assign options

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(InfiniteTree).call(this));

        _this.options = {
            autoOpen: false,
            el: null,
            rowRenderer: _renderer.defaultRowRenderer
        };
        _this.state = {
            openNodes: [],
            selectedNode: null
        };
        _this.clusterize = null;
        _this.nodebucket = {};
        _this.nodes = [];
        _this.rows = [];
        _this.scrollElement = null;
        _this.contentElement = null;

        _this.contentListener = function (evt) {
            var target = evt.target;
            var currentTarget = evt.currentTarget;


            (0, _polyfill.stopPropagation)(evt);

            if (target !== currentTarget) {
                var itemTarget = target;
                var handleToggler = false;

                while (itemTarget && itemTarget.parentElement !== currentTarget) {
                    if (itemTarget.className.indexOf('tree-toggler') >= 0) {
                        handleToggler = true;
                    }
                    itemTarget = itemTarget.parentElement;
                }

                var id = itemTarget.getAttribute('aria-id');
                var node = _this.getNodeById(id);

                // Click on the toggler to open/close a tree node
                if (handleToggler) {
                    if (_this.state.openNodes.indexOf(node) >= 0) {
                        // Close node
                        _this.closeNode(node);
                    } else {
                        _this.openNode(node);
                    }
                } else {
                    _this.selectNode(node);
                }
            }
        };

        _this.options = extend({}, _this.options, options);

        if (!_this.options.el) {
            console.error('Failed to initialize infinite-tree: el is not specified.', options);
            return _possibleConstructorReturn(_this);
        }

        _this.create();

        // Load tree data if it's provided
        if (options.data) {
            _this.loadData(options.data);
        }
        return _this;
    }

    _createClass(InfiniteTree, [{
        key: 'create',
        value: function create() {
            var _this2 = this;

            if (!this.options.el) {
                throw new Error('The element option is not specified.');
            }

            var scrollElement = document.createElement('div');
            scrollElement.className = (0, _utils.classNames)('infinite-tree', 'infinite-tree-scroll');
            var contentElement = document.createElement('div');
            contentElement.className = (0, _utils.classNames)('infinite-tree', 'infinite-tree-content');

            scrollElement.appendChild(contentElement);
            this.options.el.appendChild(scrollElement);

            this.clusterize = new _clusterize2.default({
                tag: 'div',
                rows: [],
                scrollElem: scrollElement,
                contentElem: contentElement,
                no_data_class: 'infinite-tree-no-data',
                callbacks: {
                    // Will be called right before replacing previous cluster with new one.
                    clusterWillChange: function clusterWillChange() {},
                    // Will be called right after replacing previous cluster with new one.
                    clusterChanged: function clusterChanged() {},
                    // Will be called on scrolling. Returns progress position.
                    scrollingProgress: function scrollingProgress(progress) {
                        _this2.emit('scrollProgress', progress);
                    }
                }
            });

            this.scrollElement = scrollElement;
            this.contentElement = contentElement;

            (0, _polyfill.addEventListener)(this.contentElement, 'click', this.contentListener);
        }
    }, {
        key: 'destroy',
        value: function destroy() {
            (0, _polyfill.removeEventListener)(this.contentElement, 'click', this.contentListener);

            this.clear();

            if (this.clusterize) {
                this.clusterize.destroy(true); // True to remove all data from the list
                this.clusterize = null;
            }

            // Remove all child nodes
            while (this.contentElement.firstChild) {
                this.contentElement.removeChild(this.contentElement.firstChild);
            }
            while (this.scrollElement.firstChild) {
                this.scrollElement.removeChild(this.scrollElement.firstChild);
            }
            if (this.options.el) {
                var containerElement = this.options.el;
                while (containerElement.firstChild) {
                    containerElement.removeChild(containerElement.firstChild);
                }
            }
            this.contentElement = null;
            this.scrollElement = null;
        }
    }, {
        key: 'clear',
        value: function clear() {
            this.clusterize.clear();
            this.nodebucket = {};
            this.nodes = [];
            this.rows = [];
            this.state.openNodes = [];
            this.state.selectedNode = null;
        }
        // Updates list with new data

    }, {
        key: 'update',
        value: function update() {
            this.clusterize.update(this.rows);
        }
        // Add a new node after this existing node.
        // @param {object} newNode
        // @param {object} node

    }, {
        key: 'addNodeAfter',
        value: function addNodeAfter(newNode, node) {}
        // TODO

        // Add a new node before this existing node.
        // @param {object} newNode
        // @param {object} node

    }, {
        key: 'addNodeBefore',
        value: function addNodeBefore(newNode, node) {}
        // TODO

        // Add a new node as parent of this existing node.
        // @param {object} newNode
        // @param {object} node

    }, {
        key: 'addParentNode',
        value: function addParentNode(newNode, node) {}
        // TODO

        // Add a node to this parent node. If parentNode is empty, then the new node becomes a root node.
        // @param {object} newNode The new node
        // @param {object} parentNode The parent node

    }, {
        key: 'appendNode',
        value: function appendNode(newNode, parentNode) {}
        // TODO

        // Close this node. The node must have child nodes.
        // @param {object} node
        // @return {boolean} Returns true on success, false otherwise.

    }, {
        key: 'closeNode',
        value: function closeNode(node) {
            var rowRenderer = this.options.rowRenderer;

            // Retrieve node index

            var nodeIndex = this.nodes.indexOf(node);
            if (nodeIndex < 0) {
                throw new Error('Invalid node specified: node.id=' + JSON.stringify(node.id));
            }

            // Check if the closeNode action can be performed
            if (this.state.openNodes.indexOf(node) < 0) {
                return false;
            }

            // Keep selected node unchanged if "node" is equal to "this.state.selectedNode"
            if (this.state.selectedNode && this.state.selectedNode !== node) {
                // Action:
                //   close "node.0.0"
                //
                // Tree:
                // [0] - node.0
                // [1]  - node.0.0      => next selected node (index=1, total=2)
                // [2]      node.0.0.0  => last selected node (index=2, total=0)
                // [3]      node.0.0.1
                // [4]    node.0.1
                var selectedIndex = this.nodes.indexOf(this.state.selectedNode);
                var rangeFrom = nodeIndex + 1;
                var rangeTo = nodeIndex + node.state.total;

                if (rangeFrom <= selectedIndex && selectedIndex <= rangeTo) {
                    this.selectNode(node);
                }
            }

            node.state.open = false; // Set node.state.open to false
            var openNodes = this.state.openNodes.filter(function (node) {
                return node.state.more && node.state.open;
            });
            this.state.openNodes = openNodes;

            var deleteCount = node.state.total;

            {
                // Traversing up through ancestors to subtract node.state.total
                var p = node;
                while (p) {
                    p.state.total = p.state.total - deleteCount;
                    p = p.parent;
                }
            }

            // Remove elements from an array
            this.nodes.splice(nodeIndex + 1, deleteCount);
            this.rows.splice(nodeIndex + 1, deleteCount);
            this.rows[nodeIndex] = rowRenderer(node);

            // Emit the 'closeNode' event
            this.emit('closeNode', node);

            // Updates list with new data
            this.update();

            return true;
        }
        // Get a tree node by the unique node id. This assumes that you have given the nodes in the data a unique id.
        // @param {string|number} id The unique node id. A null value will be returned if node.id not matched.

    }, {
        key: 'getNodeById',
        value: function getNodeById(id) {
            var node = (this.nodebucket[id] || [])[0];
            return node !== undefined ? node : null;
        }
        // Get the selected node. Returns the row data or null.

    }, {
        key: 'getSelectedNode',
        value: function getSelectedNode() {
            return this.state.selectedNode;
        }
        // Get the state of the tree.
        // @return {object} Returns an object that contains the ids of open nodes and selected nodes

    }, {
        key: 'getState',
        value: function getState() {}
        // TODO

        // Get the root node of the tree.

    }, {
        key: 'getTree',
        value: function getTree() {
            var tree = this.nodes.length > 0 ? this.nodes[0] : null;
            while (tree && tree.parent !== null) {
                tree = tree.parent;
            }
            return tree;
        }
        // Load data in the tree.
        // @param {object|array} data The data is a node object or array of nodes

    }, {
        key: 'loadData',
        value: function loadData() {
            var _this3 = this;

            var data = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];
            var _options = this.options;
            var autoOpen = _options.autoOpen;
            var rowRenderer = _options.rowRenderer;


            this.nodes = (0, _flattree.flatten)(data, { openAllNodes: autoOpen });

            // Construct node bucket
            this.nodebucket = {};
            this.nodes.forEach(function (node) {
                if (node.id !== undefined) {
                    var nodebucket = _this3.nodebucket[node.id];
                    _this3.nodebucket[node.id] = nodebucket ? nodebucket.concat(node) : [node];
                }
            });

            var openNodes = this.nodes.filter(function (node) {
                return node.state.more && node.state.open;
            });
            this.state.openNodes = openNodes;
            this.state.selectedNode = null;

            this.rows = this.nodes.map(function (node) {
                return rowRenderer(node);
            });

            // Updates list with new data
            this.update();
        }
        // Open this node. The node must have child nodes.
        // @param {object} node
        // @return {boolean} Returns true on success, false otherwise.

    }, {
        key: 'openNode',
        value: function openNode(node) {
            var rowRenderer = this.options.rowRenderer;

            // Retrieve node index

            var nodeIndex = this.nodes.indexOf(node);
            if (nodeIndex < 0) {
                throw new Error('Invalid node specified: node.id=' + JSON.stringify(node.id));
            }

            // Check if the openNode action can be performed
            if (this.state.openNodes.indexOf(node) >= 0) {
                return false;
            }

            node.state.open = true; // Set node.state.open to true
            var openNodes = [node].concat(this.state.openNodes); // the most recently used items first
            this.state.openNodes = openNodes;

            var nodes = (0, _flattree.flatten)(node.children, { openNodes: this.state.openNodes });
            var rows = nodes.map(function (node) {
                return rowRenderer(node);
            });

            // Insert an array inside another array
            this.nodes.splice.apply(this.nodes, [nodeIndex + 1, 0].concat(nodes));
            this.rows.splice.apply(this.rows, [nodeIndex + 1, 0].concat(rows));
            this.rows[nodeIndex] = rowRenderer(node);

            // Emit the 'openNode' event
            this.emit('openNode', node);

            // Updates list with new data
            this.update();

            return true;
        }
        // Remove node from the tree
        // @param {object} node

    }, {
        key: 'removeNode',
        value: function removeNode(node) {}
        // TODO

        // Set the current scroll position to this node.
        // @param {object} node
        // @return {number} Returns the vertical scroll position, or -1 on error.

    }, {
        key: 'scrollToNode',
        value: function scrollToNode(node) {
            // Retrieve node index
            var nodeIndex = this.nodes.indexOf(node);
            if (nodeIndex < 0) {
                return -1;
            }
            if (!this.contentElement) {
                return -1;
            }
            // Get the offset height of the first child element that contains the "tree-item" class.
            var firstChild = this.contentElement.querySelectorAll('.tree-item')[0];
            var rowHeight = firstChild && firstChild.offsetHeight || 0;
            return this.scrollTop(nodeIndex * rowHeight);
        }
        // Get/set the current vertical position of the scroll bar.
        // @param {number} [value] An integer indicating the new position to set the scroll bar to.
        // @return {number} Returns the vertical scroll position.

    }, {
        key: 'scrollTop',
        value: function scrollTop(value) {
            if (!this.scrollElement) {
                return 0;
            }
            if (value !== undefined) {
                this.scrollElement.scrollTop = Number(value);
            }
            return this.scrollElement.scrollTop;
        }
        // Select this node. You can deselect the current node by calling selectNode(null) or selectNode().
        // @param {object} node
        // @return {boolean} Returns true on success, false otherwise.

    }, {
        key: 'selectNode',
        value: function selectNode() {
            var node = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];
            var rowRenderer = this.options.rowRenderer;


            if (node === null) {
                // Deselect the current node
                if (this.state.selectedNode) {
                    var selectedNode = this.state.selectedNode;
                    var selectedIndex = this.nodes.indexOf(selectedNode);

                    selectedNode.state.selected = false;
                    this.rows[selectedIndex] = rowRenderer(selectedNode);
                    this.state.selectedNode = null;

                    // Emit the 'selectNode' event
                    this.emit('selectNode', null);

                    // Updates list with new data
                    this.update();

                    return true;
                }

                return false;
            }

            // Retrieve node index
            var nodeIndex = this.nodes.indexOf(node);
            if (nodeIndex < 0) {
                throw new Error('Invalid node specified: node.id=' + JSON.stringify(node.id));
            }

            // Select this node
            if (this.state.selectedNode !== node) {
                node.state.selected = true;
                this.rows[nodeIndex] = rowRenderer(node);
            }

            // Deselect the current node
            if (this.state.selectedNode) {
                var _selectedNode = this.state.selectedNode;
                var _selectedIndex = this.nodes.indexOf(_selectedNode);
                _selectedNode.state.selected = false;
                this.rows[_selectedIndex] = rowRenderer(_selectedNode);
            }

            if (this.state.selectedNode !== node) {
                this.state.selectedNode = node;

                // Emit the 'selectNode' event
                this.emit('selectNode', node);
            } else {
                this.state.selectedNode = null;

                // Emit the 'selectNode' event
                this.emit('selectNode', null);
            }

            // Updates list with new data
            this.update();

            return true;
        }
        // Set the state of the tree. See getState for more information.
        // @param {object} state The state object
        // @param {string} [state.openNodes] The ids of open nodes
        // @param {string} [state.selectedNode] The id of selected node

    }, {
        key: 'setState',
        value: function setState() {
            var state = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
        }
        // TODO

        // Open or close this node.

    }, {
        key: 'toggle',
        value: function toggle(node) {}
        // TODO

        // Get the tree data as string.

    }, {
        key: 'toString',
        value: function toString() {}
        // TODO

        // Update the title of a node. You can also update the data.
        // @param {object} node
        // @param {object} data The data object
        // @param {object} [data.label] The title of a node

    }, {
        key: 'updateNode',
        value: function updateNode(node, data) {
            // TODO
        }
    }]);

    return InfiniteTree;
}(_events2.default.EventEmitter);

module.exports = InfiniteTree;