'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.defaultRowRenderer = undefined;

var _utils = require('./utils');

var defaultRowRenderer = function defaultRowRenderer(node) {
    var id = node.id;
    var label = node.label;
    var children = node.children;
    var state = node.state;
    var depth = state.depth;
    var more = state.more;
    var open = state.open;
    var path = state.path;
    var total = state.total;
    var _state$selected = state.selected;
    var selected = _state$selected === undefined ? false : _state$selected;

    var childrenLength = Object.keys(children).length;

    var togglerContent = '';
    if (more && open) {
        togglerContent = '▼';
    }
    if (more && !open) {
        togglerContent = '►';
    }
    var toggler = (0, _utils.buildHTML)('a', togglerContent, {
        'class': function () {
            if (more && open) {
                return (0, _utils.classNames)('tree-toggler');
            }
            if (more && !open) {
                return (0, _utils.classNames)('tree-toggler', 'tree-closed');
            }
            return '';
        }()
    });
    var title = (0, _utils.buildHTML)('span', (0, _utils.quoteattr)(label), {
        'class': (0, _utils.classNames)('tree-title')
    });
    var treeNode = (0, _utils.buildHTML)('div', toggler + title, {
        'class': 'tree-node',
        'style': 'margin-left: ' + depth * 12 + 'px'
    });
    var treeItem = (0, _utils.buildHTML)('div', treeNode, {
        'aria-id': id,
        'aria-expanded': more && open,
        'aria-depth': depth,
        'aria-path': path,
        'aria-selected': selected,
        'aria-children': childrenLength,
        'aria-total': total,
        'class': (0, _utils.classNames)('tree-item', { 'tree-selected': selected })
    });

    return treeItem;
};

exports.defaultRowRenderer = defaultRowRenderer;